/*

    Copyright (c) 2018 - 2020 Analog Flavor EURL

    Contact us for support:
        http://www.analogflavor.com

-----------------------------------------------------------------------------
Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.

    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.

    * Neither the name of Analog Flavor EURL nor 
      the names of its contributors may be used to endorse or promote products derived from
      this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.
-----------------------------------------------------------------------------

*/


#include <wiener_mpod_lv_hv_snmp.h>

#include <string.h>
#include <math.h>
#include <stdio.h>
#ifdef WIN32
#include <Windows.h>
void sleep_ms(int ms) {
    Sleep(ms);
}
#else
#include <unistd.h>
void sleep_ms(int ms) {
    usleep(ms * 1000);
}
#endif

int checkBitIsSet(int status_flags, int bit) {
    return status_flags & (1 << bit);
}

void printCrateStatusFlags(int crate_status) {
    if (checkBitIsSet(crate_status, 0)) {
        printf("crate status flag ON is set.\n");
    }
    if (checkBitIsSet(crate_status, 1)) {
        printf("crate status flag INHIBIT is set.\n");
    }
    if (checkBitIsSet(crate_status, 2)) {
        printf("crate status flag LOCAL is set.\n");
    }
    if (checkBitIsSet(crate_status, 3)) {
        printf("crate status flag INPUT ERROR is set.\n");
    }
    if (checkBitIsSet(crate_status, 4)) {
        printf("crate status flag CHANNEL ERROR is set.\n");
    }
    if (checkBitIsSet(crate_status, 5)) {
        printf("crate status flag FAN FAILURE is set.\n");
    }
    if (checkBitIsSet(crate_status, 8)) {
        printf("crate status flag INCOMPAT is set.\n");
    }
    if (checkBitIsSet(crate_status, 9)) {
        printf("crate status flag RESET is set.\n");
    }
    if (checkBitIsSet(crate_status, 10)) {
        printf("crate status flag DERATING is set.\n");
    }
    if (checkBitIsSet(crate_status, 11)) {
        printf("crate status flag SUPPLY FAILURE is set.\n");
    }
    if (checkBitIsSet(crate_status, 12)) {
        printf("crate status flag DERATING 2 is set.\n");
    }
    if (checkBitIsSet(crate_status, 13)) {
        printf("crate status flag SUPPLY FAILURE 2 is set.\n");
    }
}

void printModuleStatusFlags(int module_status) {
    if (checkBitIsSet(module_status, 0)) {
        printf("module status flag ADJUST is set.\n");
    }
    if (checkBitIsSet(module_status, 2)) {
        printf("module status flag LIVE INSERTION is set.\n");
    }
    if (checkBitIsSet(module_status, 3)) {
        printf("module status flag HV is set.\n");
    }
    if (checkBitIsSet(module_status, 4)) {
        printf("module status flag SERVICE is set.\n");
    }
    if (checkBitIsSet(module_status, 5)) {
        printf("module status flag V LIMIT is set.\n");
    }
    if (checkBitIsSet(module_status, 6)) {
        printf("module status flag IMPUT ERROR is set.\n");
    }
    if (! checkBitIsSet(module_status, 8)) {
        printf("module status flag BAR(SUM ERROR) is set.\n");
    }
    if (! checkBitIsSet(module_status, 9)) {
        printf("module status flag BAR(RAMP) is set.\n");
    }
    if (! checkBitIsSet(module_status, 10)) {
        printf("module status flag BAR(SAFETY) is set.\n");
    }
    if (checkBitIsSet(module_status, 11)) {
        printf("module status flag EVENT is set.\n");
    }
    if (! checkBitIsSet(module_status, 12)) {
        printf("module status flag BAR(MODULE ERROR) is set.\n");
    }
    if (! checkBitIsSet(module_status, 13)) {
        printf("module status flag BAR(SUPPLY FAILURE) is set.\n");
    }
    if (! checkBitIsSet(module_status, 14)) {
        printf("module status flag BAR(MAX TEMP) is set.\n");
    }
    if (checkBitIsSet(module_status, 15)) {
        printf("module status flag KILL is set.\n");
    }
}

void printModuleEventStatusFlags(int module_event_status) {
    if (checkBitIsSet(module_event_status, 0)) {
        printf("module event status flag POWER is set.\n");
    }
    if (checkBitIsSet(module_event_status, 2)) {
        printf("module event status flag LIVE INSERTION is set.\n");
    }
    if (checkBitIsSet(module_event_status, 4)) {
        printf("module event status flag SERVICE is set.\n");
    }
    if (checkBitIsSet(module_event_status, 5)) {
        printf("module event status flag V LIMIT is set.\n");
    }
    if (checkBitIsSet(module_event_status, 6)) {
        printf("module event status flag IMPUT ERROR is set.\n");
    }
    if (checkBitIsSet(module_event_status, 10)) {
        printf("module event status flag SAFETY is set.\n");
    }
    if (checkBitIsSet(module_event_status, 13)) {
        printf("module event status flag SUPPLY FAILURE is set.\n");
    }
    if (checkBitIsSet(module_event_status, 14)) {
        printf("module event status flag MAX TEMP is set.\n");
    }
}

void printChannelStatusFlagsLV(int channel_status) {
    if (checkBitIsSet(channel_status, 0)) {
        printf("LV channel status flag ON is set.\n");
    }
    if (checkBitIsSet(channel_status, 1)) {
        printf("LV channel status flag INHIBIT is set.\n");
    }
    if (checkBitIsSet(channel_status, 2)) {
        printf("LV channel status flag MIN SENSE V is set.\n");
    }
    if (checkBitIsSet(channel_status, 3)) {
        printf("LV channel status flag MAX SENSE V is set.\n");
    }
    if (checkBitIsSet(channel_status, 4)) {
        printf("LV channel status flag MAX TERM V is set.\n");
    }
    if (checkBitIsSet(channel_status, 5)) {
        printf("LV channel status flag MAX I is set.\n");
    }
    if (checkBitIsSet(channel_status, 6)) {
        printf("LV channel status flag MAX TEMP is set.\n");
    }
    if (checkBitIsSet(channel_status, 7)) {
        printf("LV channel status flag MAX PLOAD is set.\n");
    }
    if (checkBitIsSet(channel_status, 9)) {
        printf("LV channel status flag TIMEOUT is set.\n");
    }
    if (checkBitIsSet(channel_status, 10)) {
        printf("LV channel status flag CONSTANT CURRENT is set.\n");
    }
    if (checkBitIsSet(channel_status, 11)) {
        printf("LV channel status flag RAMP UP is set.\n");
    }
    if (checkBitIsSet(channel_status, 12)) {
        printf("LV channel status flag RAMP DOWN is set.\n");
    }
}

void printChannelStatusFlagsHV(int channel_status) {
    if (checkBitIsSet(channel_status, 0)) {
        printf("HV channel status flag ON is set.\n");
    }
    if (checkBitIsSet(channel_status, 1)) {
        printf("HV channel status flag INHIBIT is set.\n");
    }
    if (checkBitIsSet(channel_status, 4)) {
        printf("HV channel status flag V LIMIT is set.\n");
    }
    if (checkBitIsSet(channel_status, 5)) {
        printf("HV channel status flag TRIP is set.\n");
    }
    if (checkBitIsSet(channel_status, 10)) {
        printf("HV channel status flag CONSTANT CURRENT is set.\n");
    }
    if (checkBitIsSet(channel_status, 11)) {
        printf("HV channel status flag RAMP UP is set.\n");
    }
    if (checkBitIsSet(channel_status, 12)) {
        printf("HV channel status flag RAMP DOWN is set.\n");
    }
    if (checkBitIsSet(channel_status, 13)) {
        printf("HV channel status flag KILL is set.\n");
    }
    if (checkBitIsSet(channel_status, 14)) {
        printf("HV channel status flag EMERGENCY is set.\n");
    }
    if (checkBitIsSet(channel_status, 15)) {
        printf("HV channel status flag ADJUST is set.\n");
    }
    if (checkBitIsSet(channel_status, 16)) {
        printf("HV channel status flag CONSTANT VOLTAGE is set.\n");
    }
    if (checkBitIsSet(channel_status, 17)) {
        printf("HV channel status flag LOW CURRENT RANGE is set.\n");
    }
    if (checkBitIsSet(channel_status, 18)) {
        printf("HV channel status flag V BOUND is set.\n");
    }
    if (checkBitIsSet(channel_status, 19)) {
        printf("HV channel status flag I LIMIT is set.\n");
    }
}

int main(int argc, char **argv) {

    void *obj_ptr;
    const char *string_value;
    int int_value;
    const int *occupied_slots;
    const int *occupied_slots_iterator;
    unsigned module_slot_index, nb_channels_per_module, channel_index;
    int thread_polling_interval_ms = 1000; /* set to 0 for debug */

    if (!mpod_snmp_winsock_startup()) {
        printf("Winsock startup failed.\n");
        getchar();
        return 1;
    }

    /* create object */
    obj_ptr = mpod_snmp_create();

    /*  establish connection */
    if (argc >= 2) {
        /* the port ans community can also specified as  "192.168.0.101:161:guru" only in the first argument. */
        mpod_snmp_crateConnect(obj_ptr, argv[1], 0, "", 0.5 * thread_polling_interval_ms, thread_polling_interval_ms, 2.0 * thread_polling_interval_ms);
    }
    else {
        mpod_snmp_crateConnect(obj_ptr, "TCPIP0::192.168.0.101::161::guru", 0, "", 0.5 * thread_polling_interval_ms, thread_polling_interval_ms, 2.0 * thread_polling_interval_ms);
    }

    /* check that the connection has been established */
    if (mpod_snmp_crateCheckConnectionIsOk(obj_ptr)) {

        /* make sure all data has been read to the buffer */
        sleep_ms(3 * thread_polling_interval_ms);

        /* get the list of slots occupied by a LV or HV module */
        /* the array is terminated by a -1 */
        occupied_slots = mpod_snmp_crateGetOccupiedSlots(obj_ptr);
        if (occupied_slots) {
            occupied_slots_iterator = occupied_slots;

            /* read and analyze crate status flags */
            int_value = mpod_snmp_crateGetSysStatus(obj_ptr);
            if (int_value < 0) {
                printf("CrateGetSysStatus() returns the error code %d\n", int_value);
            }
            else {
                printf("CrateGetSysStatus() returns 0x%x\n", int_value);
                printCrateStatusFlags(int_value);

                /* traverse the list of occupied slots */
                while (*occupied_slots_iterator >= 0) {
                    module_slot_index = *occupied_slots_iterator;

                    /* find out if the module is a HV module or not */
                    if (mpod_snmp_moduleGetIsHvModule(obj_ptr, module_slot_index)) {
                        /* read and analyze module status flags */
                        int_value = mpod_snmp_moduleGetStatus(obj_ptr, module_slot_index);
                        printf("ModuleGetStatus(%d) returns 0x%x\n", module_slot_index, int_value);
                        printModuleStatusFlags(int_value);
                        int_value = mpod_snmp_moduleGetEventStatus(obj_ptr, module_slot_index);
                        printf("ModuleGetEventStatus(%d) returns 0x%x\n", module_slot_index, int_value);
                        printModuleEventStatusFlags(int_value);

                        /* traverse all channels */
                        nb_channels_per_module = mpod_snmp_moduleGetNumberOfChannels(obj_ptr, module_slot_index);
                        for (channel_index = 0; channel_index < nb_channels_per_module; channel_index++) {
                            /* read and analyze channel status flags */
                            int_value = mpod_snmp_channelGetStatus(obj_ptr, module_slot_index, channel_index);
                            printf("ChannelGetStatus(%d, %d) returns 0x%x\n", module_slot_index, channel_index, int_value);
                            printChannelStatusFlagsHV(int_value);
                        }
                    }
                    else {
                        /* module status flags make no sense for LV modules */

                        /* traverse all channels */
                        nb_channels_per_module = mpod_snmp_moduleGetNumberOfChannels(obj_ptr, module_slot_index);
                        for (channel_index = 0; channel_index < nb_channels_per_module; channel_index++) {
                            /* read and analyze channel status flags */
                            int_value = mpod_snmp_channelGetStatus(obj_ptr, module_slot_index, channel_index);
                            printf("ChannelGetStatus(%d, %d) returns 0x%x\n", module_slot_index, channel_index, int_value);
                            printChannelStatusFlagsLV(int_value);
                        }
                    }

                    occupied_slots_iterator++;
                }
            }
        }
        else {
            /* print error messages */
            printf("CrateGetOccupiedSlots() fails.\n");
        }

    }
    else {

        /* print error messages */
        printf("CrateCheckConnectionIsOk() returns false.\n");
        int_value = mpod_snmp_crateGetConnectionStatus(obj_ptr);
        printf("CrateGetConnectionStatus() returns %d.\n", int_value);
        string_value = mpod_snmp_crateGetConnectionStatusAsString(obj_ptr);
        printf("CrateGetConnectionStatusAsString() returns %s.\n", string_value);
        string_value = mpod_snmp_crateGetErrorString(obj_ptr);
        printf("CrateGetErrorString() returns %s.\n", string_value);

    }

    /* delete object */
    mpod_snmp_delete(obj_ptr);

    mpod_snmp_winsock_cleanup();

#ifdef WIN32
    getchar();
#endif

    return 0;
}

